@@ -1,8 +1,8 @@
 /*
-  $Id: Configuration.h 9084 2011-03-23 10:57:04Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/Frog/trunk/include/frog/Configuration.h $
+  $Id: Configuration.h 13840 2012-01-02 13:47:26Z sloot $
+  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/Configuration.h $
 
-  Copyright (c) 2006 - 2011
+  Copyright (c) 2006 - 2012
   Tilburg University
 
   This file is part of frog.
