@@ -1,8 +1,8 @@
 /*
-  $Id: mbma_mod.h 13649 2011-12-06 15:14:20Z sloot $
+  $Id: mbma_mod.h 13948 2012-01-11 10:44:24Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/mbma_mod.h $
 
-  Copyright (c) 2006 - 2011
+  Copyright (c) 2006 - 2012
   Tilburg University
   
   This file is part of frog.
@@ -69,7 +69,7 @@ class Mbma {
   MBMAana inflectAndAffix( const std::vector<waStruct>& ana );
   void execute( const UnicodeString& word, 
 		const std::vector<std::string>& classes );
-  void postprocess( folia::FoliaElement * );
+  void postprocess( folia::FoliaElement *, folia::PosAnnotation * );
   int debugFlag;
   folia::FoliaElement *createFeature( char c ) const;
   void addMorph( folia::FoliaElement *, 
