@@ -1,8 +1,8 @@
 /*
-  $Id: Frog-util.cxx 13537 2011-11-10 14:36:15Z sloot $
+  $Id: Frog-util.cxx 13913 2012-01-09 11:41:38Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog-util.cxx $
 
-  Copyright (c) 2006 - 2011
+  Copyright (c) 2006 - 2012
   Tilburg University
 
   This file is part of frog
@@ -78,6 +78,7 @@ void getFileNames( const string& dirName, set<string>& fileNames ){
        }
        entry = readdir( dir );
     }
+    closedir( dir );
   }
 }
 
@@ -105,6 +106,8 @@ string escape( const string& in ){
   for ( size_t i=0; i < in.length(); ++i ){
     if ( in[i] == '\'' )
       out += "\\";
+    else if ( in[i] == '\\' )
+      out += "\\";
     out += in[i];
   }
   return out;
