@@ -1,8 +1,8 @@
 /*
-  $Id: Frog.cxx 13765 2011-12-21 13:52:37Z sloot $
+  $Id: Frog.cxx 13962 2012-01-11 14:49:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog.cxx $
 
-  Copyright (c) 2006 - 2011
+  Copyright (c) 2006 - 2012
   Tilburg University
 
   A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
@@ -50,14 +50,14 @@
 #include "config.h"
 #include "frog/Frog.h"
 #include "frog/Configuration.h"
+#include "libfolia/document.h"
+#include "libfolia/folia.h"
 #include "ucto/tokenize.h"
 #include "frog/mbma_mod.h"
 #include "frog/mblem_mod.h"
 #include "frog/mwu_chunker_mod.h"
 #include "frog/cgn_tagger_mod.h"
 #include "frog/Parser.h"
-#include "libfolia/document.h"
-#include "libfolia/folia.h"
 
 using namespace std;
 using namespace folia;
@@ -71,8 +71,11 @@ string TestFileName;
 string testDirName;
 string tmpDirName;
 string outputFileName;
+bool wantOUT;
 string XMLoutFileName;
+bool wantXML;
 string outputDirName;
+string xmlDirName;
 set<string> fileNames;
 string ProgName;
 int tpDebug = 0; //0 for none, more for more output
@@ -124,14 +127,15 @@ void usage( ) {
        << "\t============= OUTPUT OPTIONS ============================================\n"
        << "\t --tmpdir=<directory> (location to store intermediate files. Default /tmp )\n"
        << "\t -o <outputfile>	      Output to file, instead of default stdout\n"
-       << "\t --outputdir=<outputfile> Output to directory, instead of default stdout\n"
+       << "\t -X <xmlfile>      Output also to an XML file in folia format\n"
+       << "\t --outputdir=<dir> Output to dir, instead of default stdout\n"
+       << "\t --xmldir=<dir> Use 'dir' to output FoliA XML to.\n"      
        << "\t --keep-parser-files=[yes|no] keep intermediate parser files, (last sentence only)\n"
        << "\t============= OTHER OPTIONS ============================================\n"
        << "\t -h. give some help.\n"
        << "\t -V or --version . Show version info.\n"
        << "\t -d <debug level> (for more verbosity)\n"
        << "\t -Q               Enable quote detection in tokeniser\n";
-
 }
 
 //**** stuff to process commandline options *********************************
@@ -208,6 +212,7 @@ bool parse_args( TimblOpts& Opts ) {
   else {
     tmpDirName += "/";
   }
+#ifdef HAVE_DIRENT_H
   if ( !tmpDirName.empty() ){
     if ( !existsDir( tmpDirName ) ){
       *Log(theErrLog) << "temporary dir " << tmpDirName << " not readable" << endl;
@@ -215,6 +220,7 @@ bool parse_args( TimblOpts& Opts ) {
     }
     *Log(theErrLog) << "checking tmpdir: " << tmpDirName << " OK" << endl;
   }
+#endif
   if ( Opts.Find ( "testdir", value, mood)) {
 #ifdef HAVE_DIRENT_H
     doDirTest = true;
@@ -249,24 +255,54 @@ bool parse_args( TimblOpts& Opts ) {
     }
     Opts.Delete('t');
   };
+  wantOUT = false;
   if ( Opts.Find( "outputdir", value, mood)) {
     outputDirName = value;
+#ifdef HAVE_DIRENT_H
     if ( !outputDirName.empty() ){
       if ( !existsDir( outputDirName ) ){
 	*Log(theErrLog) << "output dir " << outputDirName << " not readable" << endl;
 	return false;
       }
     }
+#endif
+    wantOUT = true;
     Opts.Delete( "outputdir");
   }
   else if ( Opts.Find ('o', value, mood)) {
+    wantOUT = true;
     outputFileName = value;
     Opts.Delete('o');
   };
-  if ( Opts.Find ('X', value, mood)) {
+  wantXML = false;
+  if ( Opts.Find( "xmldir", value, mood)) {
+    xmlDirName = value;
+#ifdef HAVE_DIRENT_H
+    if ( !xmlDirName.empty() ){
+      if ( !existsDir( xmlDirName ) ){
+	*Log(theErrLog) << "XML output dir " << xmlDirName << " not readable" << endl;
+	return false;
+      }
+    }
+#endif
+    wantXML = true;
+    Opts.Delete( "xmldir");
+  }
+  else if ( Opts.Find ('X', value, mood)) {
+    wantXML = true;
     XMLoutFileName = value;
     Opts.Delete('X');
-  };
+  }
+  
+  if ( !outputFileName.empty() && !testDirName.empty() ){
+    *Log(theErrLog) << "useless -o value" << endl;
+    return false;
+  }
+  if ( !XMLoutFileName.empty() && !testDirName.empty() ){
+    *Log(theErrLog) << "useless -X value" << endl;
+    return false;
+  }
+
   if ( Opts.Find ('S', value, mood)) {
     doServer = true;
     listenport= value;
@@ -275,10 +311,6 @@ bool parse_args( TimblOpts& Opts ) {
     *Log(theErrLog) << "useless -outputdir option" << endl;
     return false;
   }
-  if ( !outputFileName.empty() && !testDirName.empty() ){
-    *Log(theErrLog) << "useless -o option" << endl;
-    return false;
-  }
 
   string tagset = configuration.lookUp( "settings", "tagger" );
   if ( tagset.empty() ){
@@ -310,7 +342,10 @@ bool parse_args( TimblOpts& Opts ) {
     *Log(theErrLog) << "no frogging without input!" << endl;
     return false;
   }
-  
+  return true;
+}
+
+bool init(){  
   if ( doServer ){
     // we use fork(). omp (GCC version) doesn't do well when omp is used
     // before the fork!
@@ -402,10 +437,6 @@ void TestSentence( FoliaElement* sent,
 		   TimerBlock& timers ){
   vector<Word*> swords = sent->words();
   if ( !swords.empty() ) {
-    if (tpDebug) {
-      // don't mangle debug output, so run 1 thread then
-      omp_set_num_threads( 1 );
-    }
     timers.tagTimer.start();
     string tagged = myCGNTagger.Classify( sent );
     timers.tagTimer.stop();
@@ -602,6 +633,7 @@ void Test( istream& IN,
 	   ostream& outStream,
 	   TimerBlock& timers,
 	   Common::Timer &frogTimer,
+	   const string& xmlOutFile,
 	   const string& tmpDir ) {
 
   string eosmarker = "";
@@ -653,21 +685,23 @@ void Test( istream& IN,
     *Log(theErrLog) << "Parsing (total)   took: " << timers.parseTimer << endl;
   }
   *Log(theErrLog) << "Frogging took:      " << frogTimer << endl;
-  if ( !XMLoutFileName.empty() ){
-    doc.save( XMLoutFileName );
-    *Log(theErrLog) << "resulting FoLiA doc saved in " << XMLoutFileName << endl;
+  if ( !xmlOutFile.empty() ){
+    doc.save( xmlOutFile );
+    *Log(theErrLog) << "resulting FoLiA doc saved in " << xmlOutFile << endl;
   }
   return;
 }
 
-void TestFile( const string& infilename, const string& outFileName) {
+void TestFile( const string& infilename, 
+	       const string& outFileName,
+	       const string& xmlOutFile ) {
   // init's are done
   TimerBlock timers;
   Common::Timer frogTimer;
   frogTimer.start();
 
   string TestFileName = ""; //untokenised!
-
+  
   ofstream outStream;
   if ( !outFileName.empty() ){
     if ( outStream.open( outFileName.c_str() ), outStream.bad() ){
@@ -680,11 +714,11 @@ void TestFile( const string& infilename, const string& outFileName) {
   ifstream IN( TestFileName.c_str() );
 
   if (!outFileName.empty()) {
-	  Test(IN, outStream, timers, frogTimer, tmpDirName );
+    Test(IN, outStream, timers, frogTimer, xmlOutFile, tmpDirName );
   } else {
-	  Test(IN, cout, timers, frogTimer, tmpDirName );
+    Test(IN, cout, timers, frogTimer, xmlOutFile, tmpDirName );
   }
-
+  
   if ( !outFileName.empty() )
     *Log(theErrLog) << "results stored in " << outFileName << endl;
 }
@@ -693,9 +727,8 @@ void TestFile( const string& infilename, const string& outFileName) {
 void TestServer( Sockets::ServerSocket &conn) {
   //by Maarten van Gompel
 
-  TimerBlock timers;
-  Common::Timer frogTimer;
-  frogTimer.start();
+  
+  
 
   try {
     while (true) {
@@ -719,7 +752,10 @@ void TestServer( Sockets::ServerSocket &conn) {
 
       *Log(theErrLog) << "Processing... " << endl;
       
-      Test(inputstream, outputstream, timers, frogTimer, tmpDirName );
+      TimerBlock timers;
+  	  Common::Timer frogTimer;
+  	  frogTimer.start();
+      Test(inputstream, outputstream, timers, frogTimer, "", tmpDirName );
       if (!conn.write( (outputstream.str()) ) || !(conn.write("READY\n"))  )
 	  throw( runtime_error( "write to client failed" ) );
 
@@ -735,7 +771,7 @@ void TestServer( Sockets::ServerSocket &conn) {
 
 int main(int argc, char *argv[]) {
   std::ios_base::sync_with_stdio(false);
-  cerr << "frog " << VERSION << " (c) ILK 1998 - 2011" << endl;
+  cerr << "frog " << VERSION << " (c) ILK 1998 - 2012" << endl;
   cerr << "Induction of Linguistic Knowledge Research Group, Tilburg University" << endl;
   ProgName = argv[0];
   cerr << "based on [" << Tokenizer::VersionName() << ", "
@@ -745,17 +781,30 @@ int main(int argc, char *argv[]) {
        << Tagger::VersionName() << "]" << endl;
   try {
     TimblOpts Opts(argc, argv);
-    if ( parse_args(Opts) ){
-      //gets a settingsfile for each component, 
-      //and starts init for that mod
-      
+        
+    if ( parse_args(Opts) &&
+	 init() ){
+      if ((tpDebug) || (doServer)) {
+      	//don't mangle debug output, so run 1 thread then.. also run in one thread in server mode, forking is too expensive for lots of small snippets
+      	omp_set_num_threads( 1 );
+      }
       if ( !fileNames.empty() ) {
 	string outPath;
+	string xmlPath;
 	if ( !outputDirName.empty() )
 	  outPath = outputDirName + "/";
+	if ( !xmlDirName.empty() )
+	  xmlPath = xmlDirName + "/";
 	set<string>::const_iterator it = fileNames.begin();
 	while ( it != fileNames.end() ){
-	  TestFile( testDirName +"/" + *it, outPath + *it + ".out" );
+	  string testName = testDirName +"/" + *it;
+	  string outName = outPath + *it + ".out";
+	  string xmlName;
+	  if ( !xmlDirName.empty() )
+	    xmlName = xmlPath + *it + ".xml";
+	  else if ( wantXML )
+	    xmlName = *it + ".xml"; // do not clobber the inputdir!
+	  TestFile( testName, outName, xmlName );
 	  ++it;
 	}
       }
@@ -764,7 +813,9 @@ int main(int argc, char *argv[]) {
 	struct sigaction action;
 	action.sa_handler = SIG_IGN;
 	sigemptyset(&action.sa_mask);
+#ifdef SA_NOCLDWAIT
 	action.sa_flags = SA_NOCLDWAIT;
+#endif	
 	sigaction(SIGCHLD, &action, NULL); 
 	
 	srand((unsigned)time(0));
@@ -803,10 +854,15 @@ int main(int argc, char *argv[]) {
 	  } catch ( std::exception& e )
 	  {
 	    std::cerr << "Server error:" << e.what() << "\nExiting.\n";
+	    exit(EXIT_FAILURE);
 	  }
 	
       } else {
-	TestFile( TestFileName, outputFileName );
+	if ( wantXML && XMLoutFileName.empty() )
+	  XMLoutFileName = TestFileName + ".xml";
+	if ( wantOUT && outputFileName.empty() )
+	  outputFileName = TestFileName + ".out";
+	TestFile( TestFileName, outputFileName, XMLoutFileName );
       }
     }
     else {
