@@ -1,8 +1,8 @@
 /*
-  $Id: Parser.cxx 13765 2011-12-21 13:52:37Z sloot $
+  $Id: Parser.cxx 13879 2012-01-05 11:16:34Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Parser.cxx $
 
-  Copyright (c) 2006 - 2011
+  Copyright (c) 2006 - 2012
   Tilburg University
 
   A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
@@ -30,6 +30,8 @@
 */
 
 #include "Python.h"
+#include <sys/types.h>
+#include <unistd.h>
 #include <cstdlib>
 #include <string>
 #include <iostream>
@@ -870,7 +872,9 @@ void Parser::prepareParse( FoliaElement *sent, parseData& pd ) {
  
 void Parser::Parse( FoliaElement *sent,
 		    const string& tmpDirName, TimerBlock& timers ){
-  fileName = tmpDirName+"csiparser";
+  pid_t pid = getpid();
+  string pids = toString( pid );
+  fileName = tmpDirName+"csi."+pids;
   timers.parseTimer.start();
   if ( !isInit ){
     *Log(parseLog) << "Parser is not initialized!" << endl;
