@@ -1,8 +1,8 @@
 /*
-  $Id: mblem_mod.cxx 13763 2011-12-21 12:07:49Z sloot $
+  $Id: mblem_mod.cxx 13948 2012-01-11 10:44:24Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mblem_mod.cxx $
 
-  Copyright (c) 2006 - 2011
+  Copyright (c) 2006 - 2012
   Tilburg University
 
   This file is part of frog.
@@ -165,7 +165,11 @@ void addAnnotation( FoliaElement *word,
 }
   
 string Mblem::postprocess( FoliaElement *word ){
-  string tag = word->pos();
+  string tag;
+#pragma omp critical(foliaupdate)
+  {
+    tag = word->pos();
+  }
   if ( debug ){
     cout << "\n\tlemmas: ";
     for( vector<mblemData>::const_iterator it=mblemResult.begin(); 
@@ -198,8 +202,13 @@ string Mblem::postprocess( FoliaElement *word ){
 } 
 
 string Mblem::Classify( FoliaElement *sword ){
-  string word = sword->str();
-  string tag = sword->annotation<PosAnnotation>()->feat("head");
+  string word;
+  string tag;
+#pragma omp critical(foliaupdate)
+  {  
+    word = sword->str();
+    tag = sword->annotation<PosAnnotation>()->feat("head");
+  }
   if ( tag == "SPEC" ) {
     addAnnotation( sword, word );
     return word;
